function x = proffsub(format,R,prof,x)
% ----------------------------------------------------------------------
% In-place forward substitution, solving R'*x = b for an upper
% triangular profile (skyline) Cholesky factor R.  R can be stored as a
% dense nxn matrix, as a band matrix or as a fully packed profile
% matrix.  In each case, prof specifies its profile shape. See
% profchol() for details. x is a dense nxk matrix, k>=1.
% ----------------------------------------------------------------------

if nargin<4, error('proffsub: missing arguments'); end
[b,n] = size(R);
nx = size(x,1);

% Skip any leading zeros in x.
for px=1:nx
   if any(x(px,:)), break; end
end

if format == 1	% unpacked n x n matrix

   for i=px:n
      J = max(px,prof(i)):i-1;
      x(i,:) = (x(i,:) - R(J,i)'*x(J,:)) / R(i,i);
   end

elseif format == 2	% packed band matrix

   % R(prof(i):b,i) represents column i elements A(i+prof(i)-b:i,i) of
   % unpacked matrix.
   for i=px:n
      J = max(px,prof(i)+i-b):i-1;
      x(i,:) = (x(i,:) - R(J-(i-b),i)'*x(J,:)) / R(b,i);
   end

elseif format == 3	% fully packed profile matrix

   % R(prof(i)+1:prof(i+1)) represents column i elements
   % A(prof(i)+1-prof(i+1)+i,i) of the unpacked matrix A.

   n = size(prof,2)-1;
   for i = px:n
      di = prof(i+1);
      bi = di-i;
      pi = prof(i)+1-bi;
      J = max(px,pi):i-1;
      x(i,:) = (x(i,:) - R(bi+J,1)'*x(J,:)) / R(di);
   end
   
else
   error('proffsub: unknown matrix format');
end
